REM *********************************************************************
REM    program: INTerpolation of SLOvenian Geoid
REM     source: INTSLOG.BAS
REM executable: INTSLOG.EXE
REM       date: 15 November 1999
REM
REM The program calculates interpolated values of geoidal heights
REM and their rms-errors for points from a file specified by the user,
REM using the bilinear (hyperbolic) method of interpolation.
REM
REM Two input files are needed:
REM
REM GEOIDSLO.BIN contains the geoid undulations of the Republic of Slovenia
REM              in binary format. Grid nodes outside Slovenia have
REM              the value 0 (zero).
REM
REM INPUT.DAT  contains the latitudes and longitudes of desired points
REM            in decimal degrees. For example, the point with coordinates
REM            B=45-45-00 and L=14-30-00 would have been represented as
REM            45.75, 14.5
REM
REM A single output file:
REM
REM OUTPUT.DAT contains the latitudes and longitudes from INPUT.DAT and
REM            interpolated values of geoid undulations and rms-errors
REM            in meters. The points outside Slovenia get the value 0.000
REM            both for the geoidal height and rms-error.
REM
REM See the sample files SAMPLIN.DAT and SAMPLOUT.DAT for details.
REM
REM authors:
REM Mr. Bosko Pribicevic and Dr. Damir Medak
REM Department of Higher Geodesy
REM Faculty of Geodesy, University of Zagreb
REM Kaciceva 26, HR-10000 Zagreb, Croatia
REM e-mail: {bpribic|dmedak}@geodet.geof.hr
REM tel.: ++385 1 4268 953
REM *******************************************************************

DEFDBL A-H, O-Z
TYPE Node
  phi AS DOUBLE
  rla AS DOUBLE
  und AS DOUBLE
  rms AS DOUBLE
END TYPE
DIM RecNode AS Node

PRINT
PRINT "+--------------------------------------------+"
PRINT "|  Interpolating the grid of Slovenian geoid |"
PRINT "|  by Bosko Pribicevic and Damir Medak       |"
PRINT "|  Faculty of Geodesy, University of Zagreb  |"
PRINT "|  November 1999                             |"
PRINT "+--------------------------------------------+"
PRINT
PRINT "Reading data from input files..."
OPEN "geoidslo.bin" FOR RANDOM AS #1 LEN = LEN(RecNode)
OPEN "input.dat" FOR INPUT AS #2
OPEN "output.dat" FOR OUTPUT AS #3

REM Limits and parameters:

nrows = 106
ncols = 141
phimin = 45.25#
rlamin = 13.25#
phimax = 47#
rlamax = 16.75#
dphi = 1# / 60#
drla = 1.5# / 60#
nout = 0
ntotal = 0
undef = 0!

REM Main loop:

WHILE NOT EOF(2)
 errc = 1
 INPUT #2, phi, rla
 phi1 = (phi - phimin) / dphi
 rla1 = (rla - rlamin) / drla
 i = FIX(phi1) + 1
 j = FIX(rla1) + 1
 i1 = i + 1
 j1 = j + 1

 IF i < 1 OR i > nrows - 1 THEN und1 = undef: rms1 = undef: GOTO 10
 IF j < 1 OR j > ncols - 1 THEN und1 = undef: rms1 = undef: GOTO 10

 k1 = (i - 1) * ncols + j
 k2 = i * ncols + j
 k3 = k1 + 1
 k4 = k2 + 1

 GET #1, k1, RecNode
   u11 = RecNode.und
   IF u11 = undef THEN errc = 0: GOTO 10
   r11 = RecNode.rms
 GET #1, k2, RecNode
   u21 = RecNode.und
   IF u21 = undef THEN errc = 0: GOTO 10
   r21 = RecNode.rms
 GET #1, k3, RecNode
   u12 = RecNode.und
   IF u12 = undef THEN errc = 0: GOTO 10
   r12 = RecNode.rms
 GET #1, k4, RecNode
   u22 = RecNode.und
   IF u22 = undef THEN errc = 0: GOTO 10
   r22 = RecNode.rms

 rn = phi1 - i + 1
 re = rla1 - j + 1
 rnm1 = 1# - rn
 rem1 = 1# - re

 und1 = rnm1 * rem1 * u11 + rn * rem1 * u21 + rnm1 * re * u12 + rn * re * u22
 rms1 = rnm1 * rem1 * r11 + rn * rem1 * r21 + rnm1 * re * r12 + rn * re * r22

10 IF errc = 0 THEN und1 = undef: rms1 = undef: nout = nout + 1
  
   ntotal = ntotal + 1
   PRINT #3, USING "##.######, ##.######, ###.###, ##.###"; phi; rla; und1; rms1
WEND

CLOSE #1
CLOSE #2
CLOSE #3

PRINT "Calculations are finished."
PRINT "Total number of points : "; ntotal
PRINT "Points outside Slovenia: "; nout
PRINT "The results are in OUTPUT.DAT"

END